@@ -166,19 +166,44 @@ char *convert_char(int uc) {
     char *mapped;
     int c;
     if ((mapped=map_subst(spec_chars,uc))) return mapped;
-    c =from_unicode(target_charset,uc);
-    if (c>=0) {
-        *plain_char=c;
-        return plain_char;
-    }
-    if ((mapped = map_subst(replacements,uc))) return mapped;
-    if (unknown_as_hex) {
-       sprintf(hexbuf,"\\x%04X",(unsigned)uc);
-       /* This sprintf is safe, becouse uc is unicode character code,
-          which cannot be greater than 0xFFFE. It is ensured by routines
-	  in reader.c
-	*/
-       return hexbuf;
-    }   
-    return  bad_char;
+    if (target_charset) { 
+	    c =from_unicode(target_charset,uc);
+	if (c>=0) {
+	    *plain_char=c;
+	    return plain_char;
+	}
+	if ((mapped = map_subst(replacements,uc))) return mapped;
+	if (unknown_as_hex) {
+	   sprintf(hexbuf,"\\x%04X",(unsigned)uc);
+	   /* This sprintf is safe, becouse uc is unicode character code,
+	      which cannot be greater than 0xFFFE. It is ensured by routines
+	      in reader.c
+	    */
+	   return hexbuf;
+	}   
+	return  bad_char;
+    } else {
+      /* NULL target charset means UTF-8 output */
+      return to_utf8(uc);
+    }  
 }
+
+char *to_utf8(unsigned int uc) {
+    static char utfbuffer[4]; /* it shouldn't overflow becouse we never deal
+			  with chars greater than 65535*/
+    int count=0;
+    if (uc< 0x80) {
+      utfbuffer[0]=uc;
+      count=1;
+    } else  {
+	if (uc < 0x800) {
+	  utfbuffer[count++]=0xC0 | (uc >> 6);
+	} else {
+	  utfbuffer[count++]=0xE0 | (uc >>12);
+	  utfbuffer[count++]=0x80 | ((uc >>6) &0x3F);
+	}	    
+        utfbuffer[count++]=0x80 | (uc & 0x3F);
+    }  
+    utfbuffer[count]=0;
+    return utfbuffer;
+}    
